# -*- coding: utf-8 -*-
#
# PyParadox is a nix launcher for Paradox titles.
# Copyright (C) 2014  Carmen Bianca Bakker <carmenbbakker@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import json

import appdirs

DEFAULT_CONFIG_DIR = appdirs.user_config_dir("PyParadox")


def default_values(game):
    mods = []
    excluded_dlcs = []

    if game == "ck2":
        full_game = "Crusader Kings II"
    elif game == "eu4":
        full_game = "Europa Universalis IV"
    else:
        full_game = ""

    # Windows
    if sys.platform.startswith("win"):
        binary_path = os.path.join(os.environ["ProgramFiles(x86)"],
                                   "Steam",
                                   "SteamApps",
                                   "common",
                                   full_game,
                                   "{}.exe".format(game))
        mod_path = os.path.join(os.path.expanduser("~"),
                                "Documents",
                                "Paradox Interactive",
                                full_game,
                                "mod")
    # OS X
    elif sys.platform == 'darwin':
        binary_path = os.path.join(appdirs.user_data_dir("Steam"),
                                   "SteamApps",
                                   "common",
                                   full_game,
                                   game)
        mod_path = os.path.join(os.path.expanduser("~"),
                                "Documents",
                                "Paradox Interactive",
                                full_game,
                                "mod")
    # Linux
    else:
        binary_path = os.path.join(os.path.expanduser("~"),
                                   ".local",
                                   "share",
                                   "Steam",
                                   "steamapps",
                                   "common",
                                   full_game,
                                   game)
        mod_path = os.path.join(os.path.expanduser("~"),
                                ".paradoxinteractive",
                                full_game,
                                "mod")

    return {
        "game": game,
        "mods": mods,
        "excluded_dlcs": excluded_dlcs,
        "binary_path": binary_path,
        "mod_path": mod_path
    }


class JsonBackend:

    @staticmethod
    def read(stream):
        return json.load(stream)

    @staticmethod
    def write(data, stream):
        json.dump(data, stream, indent=4, sort_keys=True)


class Config(dict):

    def __init__(self, *args, **kwargs):
        self.backend = kwargs.pop("backend", JsonBackend)
        super().__init__(*args, **kwargs)

    @staticmethod
    def build_config(stream_or_file, default=None, backend=JsonBackend):
        try:
            stream = open(stream_or_file)
        except TypeError:
            stream = stream_or_file
        except FileNotFoundError:
            stream = None

        if stream:
            data = backend.read(stream)
            stream.close()
        else:
            data = default if default is not None else dict()

        return Config(data, backend=backend)

    def load(self, stream, override=True, suppress_merge=False):
        data = self.backend.read(stream)
        if override:
            self.update(data)
        else:
            for key, value in data.items():
                if key in self:
                    if not suppress_merge:
                        raise AttributeError("key '{}' already exists".format(
                            key))
                else:
                    self[key] = value

    def save(self, stream):
        self.backend.write(self, stream)


class FileConfig(Config):

    def __init__(self, path, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.path = path

    @staticmethod
    def build_config(path, default=None, backend=JsonBackend):
        config = Config.build_config(path, default=default, backend=backend)

        return FileConfig(path, dict(config.items()), backend=backend)

    def load(self, override=True, suppress_merge=True):
        with open(self.path):
            super().load(self.path, override=override,
                         suppress_merge=suppress_merge)

    def save(self):
        _mkdir_p(os.path.dirname(self.path))

        with open(self.path, "w") as fp:
            super().save(fp)


def _mkdir_p(directory):
    try:
        os.makedirs(directory)
    except FileExistsError:
        pass



# -*- coding: utf-8 -*-
#
# PyParadox is a nix launcher for Paradox titles.
# Copyright (C) 2014  Carmen Bianca Bakker <carmenbbakker@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import subprocess
import glob
import re

GAMES = {"ck2": "Crusader Kings II", "eu4": "Europa Universalis IV"}
EXPANSION_NAMES = ["Charlemagne", "Legacy of Rome", "Rajas of India",
                   "Sons of Abraham", "Sunset Invasion", "The Old Gods",
                   "The Republic", "The Sword of Islam", "Way of Life",
                   "Art of War", "Res Publica", "Wealth of Nations",
                   "El Dorado"]


class Plugin(object):
    """Represents a single plugin (mod/DLC/expansion)."""

    MOD = 0
    DLC = 1
    EXPANSION = 2

    def __init__(self, name, file_name, plugin_type=0, enabled=True):

        self.name = name
        self.file_name = file_name
        self.plugin_type = plugin_type
        self.enabled = enabled


def find_plugins(path, extension, config=None):
    extension = extension.strip(".")
    plugin_paths = glob.glob(os.path.join(path, "*.{}".format(extension)))

    plugins = []

    if config is not None:
        mods = config["mods"]
        excluded_dlcs = config["excluded_dlcs"]
    else:
        mods = []
        excluded_dlcs = []

    for path in plugin_paths:
        file_name = os.path.basename(path)

        with open(path) as fp:
            try:
                name = find_plugin_name(fp.read())
            except ValueError:
                name = file_name

        if extension == "mod":
            plugin_type = Plugin.MOD
            enabled = file_name in mods
        elif extension == "dlc":
            plugin_type = (Plugin.EXPANSION if name in EXPANSION_NAMES else
                           Plugin.DLC)
            enabled = file_name not in excluded_dlcs
        else:
            # Assume it is a mod. This technically should never happen.
            plugin_type = Plugin.MOD
            enabled = file_name in mods

        plugin = Plugin(name, file_name, plugin_type=plugin_type,
                        enabled=enabled)
        plugins.append(plugin)

    return plugins


def find_plugin_name(plugin_contents):
    try:
        return re.search('^name[ \t]*=[ \t]*"(.*)"', plugin_contents,
                         re.MULTILINE).group(1)
    except AttributeError as e:
        raise ValueError("Could not find plugin name",
                         plugin_contents) from e


def make_command(binary_path, mods, excluded_dlcs):
    command = [binary_path]
    command.append("-skiplauncher")
    for mod in mods:
        command.append("-mod={}".format(os.path.join("mod", mod)))
    for dlc in excluded_dlcs:
        command.append("-exclude_dlc={}".format(os.path.join("dlc", dlc)))
    return command


def execute_command(command, synchronous=False):
    if synchronous:
        return subprocess.call(command)
    else:
        return subprocess.Popen(command)



# -*- coding: utf-8 -*-
#
# PyParadox is a nix launcher for Paradox titles.
# Copyright (C) 2014  Carmen Bianca Bakker <carmenbbakker@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import argparse

from .config import default_values, FileConfig, DEFAULT_CONFIG_DIR
from .core import GAMES, make_command, execute_command
from . import __version__


def main(game="ck2"):
    # main_console(["ck2"])
    main_gui([game])


def main_console(args=sys.argv):
    parser_args = _parse_args(args=args)
    game = parser_args.game
    path = os.path.join(DEFAULT_CONFIG_DIR, "pyparadox_{}.json".format(game))
    config = FileConfig.build_config(path, default=default_values(game))

    command = make_command(config["binary_path"],
                           config["excluded_dlcs"],
                           config["mods"])
    execute_command(command)


def main_gui(args=sys.argv):
    parser_args = _parse_args(args=args)
    game = parser_args.game
    path = os.path.join(DEFAULT_CONFIG_DIR, "pyparadox_{}.json".format(game))

    config = FileConfig.build_config(path, default=default_values(game))

    from PyQt5 import QtWidgets
    from .gui import MainWindow

    app = QtWidgets.QApplication(sys.argv)
    app.setApplicationVersion(__version__)
    app.setApplicationName("PyParadox")

    form = MainWindow(config)
    form.show()

    sys.exit(app.exec_())


def _parse_args(args=sys.argv):
    parser = argparse.ArgumentParser(description="Launcher for Paradox titles")
    parser.add_argument("game", type=str, choices=list(GAMES.keys()),
                        help="Game to launch")
    return parser.parse_args(sys.argv[1:])


if __name__ == "__main__":
    main()
